;;; $Id: index.lisp 403 2003-08-07 03:56:18Z miles $

(in-package :macho)

(defun index-load (archive)
  (msg "loading index for ~S~%" (archive-name archive))
  (let ((index-path (archive-index-path archive)))
    (setf (archive-messages archive) (make-array 0 :fill-pointer t))
    (if (probe-file index-path)
        (with-open-file (s index-path :direction :input :element-type :default)
          (loop with version = (read-int32 s)
                with last-pos = (read-int32 s)
                with count = (read-int32 s)
                for i from 1 to count
                for msg = (make-instance 'message)
                do (with-slots (start end time serial id author subject in-reply) msg
                     (setf (message-archive msg) archive
                           start (read-int32 s)
                           end (read-int32 s)
                           time (time-struct (read-int32 s))
                           serial i
                           id (read-tagged-string s)
                           author (read-tagged-string s)
                           subject (read-tagged-string s)
                           in-reply (read-tagged-string s))
                     (vector-push-extend msg (archive-messages archive)))
                finally (setf (archive-last-pos archive) last-pos))))))

(defun index-save (archive)
  (msg "saving index for ~S~%" (archive-name archive))
  (with-open-file (s (archive-index-path archive) :direction :output :element-type :default)
    (let ((messages (archive-messages archive)))
      (write-int32 1 s) ;; file format version
      (write-int32 (archive-last-pos archive) s)
      (write-int32 (length messages) s)
      (loop for m across messages
            do (with-slots (start end time id author subject in-reply) m
                 (write-int32 start s)
                 (write-int32 end s)
                 (write-int32 (time-universal time) s)
                 (write-tagged-string id s)
                 (write-tagged-string author s)
                 (write-tagged-string subject s)
                 (write-tagged-string in-reply s))))))

(defun index-update (archive)
  (msg "updating index for ~S~%" (archive-name archive))
  (with-slots (mbox mbox-path index-path last-pos messages) archive
    (index-load archive)
    (if (or (not (probe-file index-path))
            (< (file-write-date index-path) (file-write-date mbox-path)))
        (progn
          (msg "adding to index for ~S from mbox~%" (archive-name archive))
          (file-position mbox last-pos)
          (loop with next = (length messages)
                for start-pos = (file-position mbox)
                for txt = (read-delimited mbox #.(format nil "~CFrom " #\Newline))
                for end-pos = (file-position mbox)
                for msg = (parse-message txt)
                while txt do (with-slots (start end serial dirty) msg
                               (setf dirty t
                                     (message-archive msg) archive
                                     serial (incf next)
                                     start start-pos
                                     end end-pos)
                               (msg "read message ~A~%" next)
                               (vector-push-extend msg messages))
                finally (setf last-pos end-pos))))
    (index-save archive)))
